/* controllable.c generated by valac 0.16.0, the Vala compiler
 * generated from controllable.vala, do not modify */

/**
         * Set the presentation controller which needs to be informed of key
         * presses and such.
         */

#include <glib.h>
#include <glib-object.h>


#define PDFPC_TYPE_CONTROLLABLE (pdfpc_controllable_get_type ())
#define PDFPC_CONTROLLABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PDFPC_TYPE_CONTROLLABLE, pdfpcControllable))
#define PDFPC_IS_CONTROLLABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PDFPC_TYPE_CONTROLLABLE))
#define PDFPC_CONTROLLABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), PDFPC_TYPE_CONTROLLABLE, pdfpcControllableIface))

typedef struct _pdfpcControllable pdfpcControllable;
typedef struct _pdfpcControllableIface pdfpcControllableIface;

#define PDFPC_TYPE_PRESENTATION_CONTROLLER (pdfpc_presentation_controller_get_type ())
#define PDFPC_PRESENTATION_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PDFPC_TYPE_PRESENTATION_CONTROLLER, pdfpcPresentationController))
#define PDFPC_PRESENTATION_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PDFPC_TYPE_PRESENTATION_CONTROLLER, pdfpcPresentationControllerClass))
#define PDFPC_IS_PRESENTATION_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PDFPC_TYPE_PRESENTATION_CONTROLLER))
#define PDFPC_IS_PRESENTATION_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PDFPC_TYPE_PRESENTATION_CONTROLLER))
#define PDFPC_PRESENTATION_CONTROLLER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PDFPC_TYPE_PRESENTATION_CONTROLLER, pdfpcPresentationControllerClass))

typedef struct _pdfpcPresentationController pdfpcPresentationController;
typedef struct _pdfpcPresentationControllerClass pdfpcPresentationControllerClass;

struct _pdfpcControllableIface {
	GTypeInterface parent_iface;
	pdfpcPresentationController* (*get_controller) (pdfpcControllable* self);
	void (*update) (pdfpcControllable* self);
	void (*edit_note) (pdfpcControllable* self);
	void (*ask_goto_page) (pdfpcControllable* self);
	void (*show_overview) (pdfpcControllable* self);
	void (*hide_overview) (pdfpcControllable* self);
};



GType pdfpc_presentation_controller_get_type (void) G_GNUC_CONST;
GType pdfpc_controllable_get_type (void) G_GNUC_CONST;
pdfpcPresentationController* pdfpc_controllable_get_controller (pdfpcControllable* self);
void pdfpc_controllable_update (pdfpcControllable* self);
void pdfpc_controllable_edit_note (pdfpcControllable* self);
void pdfpc_controllable_ask_goto_page (pdfpcControllable* self);
void pdfpc_controllable_show_overview (pdfpcControllable* self);
void pdfpc_controllable_hide_overview (pdfpcControllable* self);


/**
         * Return the registered PresentationController
         */
pdfpcPresentationController* pdfpc_controllable_get_controller (pdfpcControllable* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return PDFPC_CONTROLLABLE_GET_INTERFACE (self)->get_controller (self);
}


/**
         * Update the display
         */
void pdfpc_controllable_update (pdfpcControllable* self) {
	g_return_if_fail (self != NULL);
	PDFPC_CONTROLLABLE_GET_INTERFACE (self)->update (self);
}


/**
         * Edit note for current slide
         */
void pdfpc_controllable_edit_note (pdfpcControllable* self) {
	g_return_if_fail (self != NULL);
	PDFPC_CONTROLLABLE_GET_INTERFACE (self)->edit_note (self);
}


/**
         * Ask for the page to jump to
         */
void pdfpc_controllable_ask_goto_page (pdfpcControllable* self) {
	g_return_if_fail (self != NULL);
	PDFPC_CONTROLLABLE_GET_INTERFACE (self)->ask_goto_page (self);
}


/**
         * Show an overview of all slides
         */
void pdfpc_controllable_show_overview (pdfpcControllable* self) {
	g_return_if_fail (self != NULL);
	PDFPC_CONTROLLABLE_GET_INTERFACE (self)->show_overview (self);
}


/**
         * Hide the overview
         */
void pdfpc_controllable_hide_overview (pdfpcControllable* self) {
	g_return_if_fail (self != NULL);
	PDFPC_CONTROLLABLE_GET_INTERFACE (self)->hide_overview (self);
}


static void pdfpc_controllable_base_init (pdfpcControllableIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
	}
}


/**
     * Every window or object which wants to be controlled by the
     * PresentationController needs to implement this interface.
     */
GType pdfpc_controllable_get_type (void) {
	static volatile gsize pdfpc_controllable_type_id__volatile = 0;
	if (g_once_init_enter (&pdfpc_controllable_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (pdfpcControllableIface), (GBaseInitFunc) pdfpc_controllable_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType pdfpc_controllable_type_id;
		pdfpc_controllable_type_id = g_type_register_static (G_TYPE_INTERFACE, "pdfpcControllable", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (pdfpc_controllable_type_id, G_TYPE_OBJECT);
		g_once_init_leave (&pdfpc_controllable_type_id__volatile, pdfpc_controllable_type_id);
	}
	return pdfpc_controllable_type_id__volatile;
}



